--****************************************************************--
--* ALU
--*
--* I/O Configuration:
--* Input: A, B, Control Line
--* Output: F
--*
--* FUNCTIONS AND CONTROL:
--* Control Line: S0, S1, S2
--* -----------------------------------------------------------
--*  Control Line   | Micro-             |
--*  S0   S1   S2   | Operation          | Output Result
--* -----------------------------------------------
--*   0    0    0   | INCA               | F <- A
--*   0    0    1   | DECA               | F <- A + 1
--*   0    1    0   | ADD                | F <- A + B
--*   0    1    0   | ADC                | F <- A + B + 1
--*   0    1    1   | SUB                | F <- A - B
--*   0    1    1   | SUBC               | F <- A - B - 1
--*   1    0    0   | OR                 | F <- A or B
--*   1    0    1   | XOR                | F <- A xor B
--*   1    1    0   | AND                | F <- A and B
--*   1    1    1   | NOT                | F <- A'
--****************************************************************--

Library IEEE;
	use IEEE.std_logic_1164.all;
	use IEEE.std_logic_arith.all;
	use IEEE.std_logic_signed.all;
	
entity ALU is
	port(	A, B: in std_logic_vector(7 downto 0);  -- Input A and B
			S: in std_logic_vector(2 downto 0);     -- Control Lines
			F: out std_logic_vector(7 downto 0)     -- Output Result
		 );
end ALU;

architecture BEHAVE_ALU of ALU is
begin	
	process(A, B, S)
	begin
		case S is
			when "000" => 	F <= A + 1;		-- Increment
			when "001" => 	F <= A - 1;		-- Decrement
			when "010" => 	F <= A + B;		-- Addition
			when "011" => 	F <= A - B;		-- Subtraction
			when "100" => 	F <= A or B;	-- Logic OR
			when "101" =>	F <= A xor B;	-- Logic XOR
			when "110" =>	F <= A and B;	-- Logic AND
			when "111" =>	F <= NOT(A);	-- Logic Invert
			when others => F <= "XXXXXXXX";		-- Undefine Control Signal
		end case;
	end process;
end BEHAVE_ALU;
			
